'use strict';
var Promise = require('bluebird');
var errorObject = {value: null};
function tryCatch(fn, ctx, args) {
  try {
    return fn.apply(ctx, args);
  }
  catch(e) {
    errorObject.value = e;
    return errorObject;
  }
}

function isEmpty(obj){
  for(var key in obj) {
    if(obj.hasOwnProperty(key)){
      return false;
    }
  }
  return true;
}

/**
 * Waits for a redis client to be ready.
 * @param {Redis} redis client
 */
function isRedisReady(client){
  return new Promise(function(resolve, reject){
    if(client.status === 'ready'){
      resolve();
    } else {
      function handleReady(){
        client.removeListener('error', handleError);
        resolve();
      }

      function handleError(err){
        client.removeListener('ready', handleError);
        reject(err);
      }

      client.once('ready', handleReady);
      client.once('error', handleError);
    }
  });
}

module.exports.errorObject = errorObject;
module.exports.tryCatch = tryCatch;
module.exports.isEmpty = isEmpty;
module.exports.isRedisReady = isRedisReady;
